import time
from calendar import monthrange

from netforce.model import Model, fields, get_model
from netforce.utils import get_file_path
from . import utils

class MatchingHDCase(Model):
    _name="clinic.matching.hdcase"
    _transient=True

    _fields={
        "date": fields.Date("Date", required=True),
        'file': fields.File("File"),
    }
    
    def _get_date(self,context={}):
        return time.strftime("%Y-%m-%d")

    _defaults={
        'date': _get_date,
    }
    
    def get_rows(self,fpath=None):
        if not fpath:
            raise Exception("File not found")
        try:
            # or codecs.open on Python 2
            filedata = open(fpath, encoding='UTF-8').read() 
        except:
            filedata = open(fpath, encoding='TIS-620').read() 
        lines=filedata.split("\n")
        count=0
        rows=[]
        for line in lines:
            #open
            if '***' in line and count==0:
                count+=1
            #end    
            elif '***' in line and count==1:
                count=0

            #table content
            if count==1:
                line=line.replace("*","")
                line=line.replace("|","")
                i=0
                vals={}
                for l in line.split(","):
                    if l:
                        vals[i]=l  
                        i+=1
                # only right data
                if vals:
                    if len(vals)>1:
                        rows.append(vals)
        return rows

    def get_report_data(self,ids,context={}):
        lines=[]
        hdcases={}
        if ids:
            obj=self.browse(ids)[0]
            if obj.file:
                fpath=get_file_path(obj.file)
                rows=self.get_rows(fpath)
                if not rows:
                    raise Exception("No Data")
                #{0: ' A 01',
                 #1: ' 6',
                 #2: ' 11686',
                 #3: ' 91312851',
                 #4: ' 450124497',
                 #5: ' 27/01/2558 06:05:00',
                 #6: ' C',
                 #7: ' O',
                 #8: ' U ',
                 #9: ' N',
                 #10: ' Espogen',
                 #11: '       4000',
                 #12: '         30',
                 #13: '          1',
                 #14: '          0',
                 #15: '       1500',
                 #16: '       1500',
                 #17: ' 0',
                 #18: '       1500',
                 #19: ' S',
                 #20: ' U '}

                patients={}
                for pt in get_model("clinic.patient").search_read([],['name','hn_no']):
                    hn=pt['hn_no']
                    patients[hn]={
                        'id': pt['id'],
                        'name': pt['name'] or '',
                    }
                products={}
                for prod in get_model("product").search_read([],['name']):
                    name=(prod['name'] or "").replace(" ", "")
                    name=name.upper()
                    products[name]=prod['id']
                dom=[]
                dom.append(['date',">=",obj.date])
                dom.append(['date',"<=",obj.date])
                dom.append(['state','!=','cancelled'])
                hdcases={}
                for hdcase in get_model("clinic.hd.case").search_browse(dom):
                    date=hdcase.date
                    hn=hdcase.patient_id.hn_no or 0
                    fee_amt=0
                    for line in hdcase.lines:
                        prod=line.product_id 
                        categ=prod.categ_id
                        prod_line=[]
                        if categ:
                            if categ.code=='EPO':
                                prod_line.append(prod.name)
                            elif categ.code=='FEE':
                                fee_amt=line.amount or 0
                    prod_name=','.join(prod_line)
                    key='%s-%s-%s-%s'%(date,hn,prod_name,fee_amt)
                    hdcases[key]={
                        'id': hdcase.id,
                        'number': hdcase.number or ''
                    }
                for row in rows:
                    vals=list(row.values())
                    date,time=(vals[5] or "").split()
                    if len(date)<3:
                        raise Exception("Wrong format date")
                    d,m,y=date.split("/")
                    y=int(y)-543
                    date='%s-%s-%s'%(y,m,d)
                    hn=vals[3]
                    hn=''.join(x for x in (hn or "") if x.isdigit())
                    pt=patients.get(hn)
                    pt_name=''
                    pt_id=None
                    if pt:
                        pt_name=pt['name']
                        pt_id=pt['id']
                    hct=vals[12] or 0
                    fee_amt=float(vals[15] or "0")
                    prod_name=vals[10] or ''
                    prod_name=prod_name.replace(" ", "")
                    prod_id=products.get(prod_name.upper(),None)
                    hd_case_id=None
                    hd_case_number=""
                    key='%s-%s-%s-%s'%(date,hn,prod_name,fee_amt)
                    if hn=='10073353':
                        print('key ', key)
                        #import pdb; pdb.set_trace()
                    hdcase=hdcases.get(key)
                    if hdcase:
                        hd_case_id=hdcase['id']
                        hd_case_number=hdcase['number']
                    vals={
                        'hn': hn,
                        'date': date,
                        'patient_name': pt_name,
                        'patient_id': pt_id,
                        'hct': hct,
                        'prod_name': prod_name, 
                        'prod_id': prod_id,
                        'fee_amt': fee_amt,
                        'hd_case_id': hd_case_id,
                        'hd_case_number': hd_case_number,
                    }
                    lines.append(vals)
        no=1
        lines2=[]
        for line in sorted(lines,key=lambda x: x['hn']):
            line['no']=no
            lines2.append(line)
            no+=1
        lines=lines2
        data={
            'lines': lines,
        }
        return data

MatchingHDCase.register()
